# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from .remote_data import RemoteData
from .scheduled_interview_status import ScheduledInterviewStatus


class ScheduledInterview(pydantic.BaseModel):
    """
    # The ScheduledInterview Object
    ### Description
    The `ScheduledInterview` object is used to represent a scheduled interview for a given candidate’s application to a job. An `Application` can have multiple `ScheduledInterview`s depending on the particular hiring process.
    ### Usage Example
    Fetch from the `LIST ScheduledInterviews` endpoint and filter by `interviewers` to show all office locations.
    """

    id: typing.Optional[str]
    remote_id: typing.Optional[str] = pydantic.Field(description="The third-party API ID of the matching object.")
    application: typing.Optional[str] = pydantic.Field(description="The application being interviewed.")
    job_interview_stage: typing.Optional[str] = pydantic.Field(description="The stage of the interview.")
    organizer: typing.Optional[str] = pydantic.Field(description="The user organizing the interview.")
    interviewers: typing.Optional[typing.List[typing.Optional[str]]] = pydantic.Field(
        description="Array of `RemoteUser` IDs."
    )
    location: typing.Optional[str] = pydantic.Field(description="The interview's location.")
    start_at: typing.Optional[str] = pydantic.Field(description="When the interview was started.")
    end_at: typing.Optional[str] = pydantic.Field(description="When the interview was ended.")
    remote_created_at: typing.Optional[str] = pydantic.Field(
        description="When the third party's interview was created."
    )
    remote_updated_at: typing.Optional[str] = pydantic.Field(
        description="When the third party's interview was updated."
    )
    status: typing.Optional[ScheduledInterviewStatus] = pydantic.Field(
        description=(
            "The interview's status.\n"
            "\n"
            "* `SCHEDULED` - SCHEDULED\n"
            "* `AWAITING_FEEDBACK` - AWAITING_FEEDBACK\n"
            "* `COMPLETE` - COMPLETE\n"
        )
    )
    remote_was_deleted: typing.Optional[bool] = pydantic.Field(
        description="Indicates whether or not this object has been deleted by third party webhooks."
    )
    modified_at: typing.Optional[str] = pydantic.Field(
        description="This is the datetime that this object was last updated by Merge"
    )
    field_mappings: typing.Optional[typing.Dict[str, typing.Any]]
    remote_data: typing.Optional[typing.List[RemoteData]]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        json_encoders = {dt.datetime: serialize_datetime}
